@@ -11,7 +11,7 @@ use uv_client::RegistryClientBuilder;
 
 #[tokio::test]
 async fn remote_metadata_with_and_without_cache() -> Result<()> {
-    let cache = Cache::temp()?;
+    let cache = Cache::temp()?.init()?;
     let client = RegistryClientBuilder::new(cache).build();
 
     // The first run is without cache (the tempdir is empty), the second has the cache from the
@@ -22,7 +22,6 @@ async fn remote_metadata_with_and_without_cache() -> Result<()> {
         let dist = BuiltDist::DirectUrl(DirectUrlBuiltDist {
             filename,
             location: Url::parse(url).unwrap(),
-            subdirectory: None,
             url: VerbatimUrl::from_str(url).unwrap(),
         });
         let metadata = client.wheel_metadata(&dist).await.unwrap();
