@@ -1,8 +1,8 @@
-use pep440_rs::{Operator, Version, VersionSpecifier, VersionSpecifiers};
-use pep508_rs::{MarkerExpression, MarkerTree, MarkerValueVersion, StringVersion};
+use pep440_rs::VersionSpecifiers;
+use pep508_rs::{MarkerTree, StringVersion};
 use uv_toolchain::{Interpreter, PythonVersion};
 
-use crate::RequiresPython;
+use crate::{RequiresPython, RequiresPythonBound};
 
 #[derive(Debug, Clone, Eq, PartialEq)]
 pub struct PythonRequirement {
@@ -49,6 +49,19 @@ impl PythonRequirement {
         }
     }
 
+    /// Narrow the [`PythonRequirement`] to the given version, if it's stricter (i.e., greater)
+    /// than the current `Requires-Python` minimum.
+    pub fn narrow(&self, target: &RequiresPythonBound) -> Option<Self> {
+        let Some(PythonTarget::RequiresPython(requires_python)) = self.target.as_ref() else {
+            return None;
+        };
+        let requires_python = requires_python.narrow(target)?;
+        Some(Self {
+            installed: self.installed.clone(),
+            target: Some(PythonTarget::RequiresPython(requires_python)),
+        })
+    }
+
     /// Return the installed version of Python.
     pub fn installed(&self) -> &StringVersion {
         &self.installed
@@ -62,32 +75,12 @@ impl PythonRequirement {
     /// Return a [`MarkerTree`] representing the Python requirement.
     ///
     /// See: [`RequiresPython::to_marker_tree`]
-    pub fn to_marker_tree(&self) -> MarkerTree {
-        let version = match &self.target {
-            None => self.installed.version.clone(),
-            Some(PythonTarget::Version(version)) => version.version.clone(),
-            Some(PythonTarget::RequiresPython(requires_python)) => {
-                return requires_python.to_marker_tree()
-            }
-        };
-
-        let version_major_minor_only = Version::new(version.release().iter().take(2));
-        let expr_python_version = MarkerExpression::Version {
-            key: MarkerValueVersion::PythonVersion,
-            specifier: VersionSpecifier::from_version(
-                Operator::GreaterThanEqual,
-                version_major_minor_only,
-            )
-            .unwrap(),
-        };
-        let expr_python_full_version = MarkerExpression::Version {
-            key: MarkerValueVersion::PythonFullVersion,
-            specifier: VersionSpecifier::from_version(Operator::GreaterThanEqual, version).unwrap(),
-        };
-        MarkerTree::And(vec![
-            MarkerTree::Expression(expr_python_version),
-            MarkerTree::Expression(expr_python_full_version),
-        ])
+    pub fn to_marker_tree(&self) -> Option<MarkerTree> {
+        if let Some(PythonTarget::RequiresPython(requires_python)) = self.target.as_ref() {
+            Some(requires_python.to_marker_tree())
+        } else {
+            None
+        }
     }
 }
 
