@@ -1,9 +1,12 @@
+use std::cmp::Ordering;
 use std::collections::Bound;
+use std::ops::Deref;
 
 use itertools::Itertools;
 use pubgrub::range::Range;
 
-use pep440_rs::{Version, VersionSpecifier, VersionSpecifiers};
+use pep440_rs::{Operator, Version, VersionSpecifier, VersionSpecifiers};
+use pep508_rs::{MarkerExpression, MarkerTree, MarkerValueVersion};
 
 #[derive(thiserror::Error, Debug)]
 pub enum RequiresPythonError {
@@ -23,7 +26,7 @@ pub enum RequiresPythonError {
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct RequiresPython {
     specifiers: VersionSpecifiers,
-    bound: Bound<Version>,
+    bound: RequiresPythonBound,
 }
 
 impl RequiresPython {
@@ -33,7 +36,7 @@ impl RequiresPython {
             specifiers: VersionSpecifiers::from(VersionSpecifier::greater_than_equal_version(
                 version.clone(),
             )),
-            bound: Bound::Included(version),
+            bound: RequiresPythonBound(Bound::Included(version)),
         }
     }
 
@@ -60,11 +63,13 @@ impl RequiresPython {
         };
 
         // Extract the lower bound.
-        let bound = range
-            .iter()
-            .next()
-            .map(|(lower, _)| lower.clone())
-            .unwrap_or(Bound::Unbounded);
+        let bound = RequiresPythonBound(
+            range
+                .iter()
+                .next()
+                .map(|(lower, _)| lower.clone())
+                .unwrap_or(Bound::Unbounded),
+        );
 
         // Convert back to PEP 440 specifiers.
         let specifiers = range
@@ -75,6 +80,16 @@ impl RequiresPython {
         Ok(Some(Self { specifiers, bound }))
     }
 
+    /// Narrow the [`RequiresPython`] to the given version, if it's stricter (i.e., greater) than
+    /// the current target.
+    pub fn narrow(&self, target: &RequiresPythonBound) -> Option<Self> {
+        let target = VersionSpecifiers::from(VersionSpecifier::from_lower_bound(target)?);
+        Self::union(std::iter::once(&target))
+            .ok()
+            .flatten()
+            .filter(|next| next.bound > self.bound)
+    }
+
     /// Returns `true` if the `Requires-Python` is compatible with the given version.
     pub fn contains(&self, version: &Version) -> bool {
         self.specifiers.contains(version)
@@ -139,7 +154,7 @@ impl RequiresPython {
         // Alternatively, we could vary the semantics depending on whether or not the user included
         // a pre-release in their specifier, enforcing pre-release compatibility only if the user
         // explicitly requested it.
-        match (target, &self.bound) {
+        match (target, self.bound.as_ref()) {
             (Bound::Included(target_lower), Bound::Included(requires_python_lower)) => {
                 target_lower.release() <= requires_python_lower.release()
             }
@@ -165,9 +180,67 @@ impl RequiresPython {
         &self.specifiers
     }
 
-    /// Returns the lower [`Bound`] for the `Requires-Python` specifier.
-    pub fn bound(&self) -> &Bound<Version> {
-        &self.bound
+    /// Returns `true` if the `Requires-Python` specifier is unbounded.
+    pub fn is_unbounded(&self) -> bool {
+        self.bound.as_ref() == Bound::Unbounded
+    }
+
+    /// Returns this `Requires-Python` specifier as an equivalent marker
+    /// expression utilizing the `python_version` marker field.
+    ///
+    /// This is useful for comparing a `Requires-Python` specifier with
+    /// arbitrary marker expressions. For example, one can ask whether the
+    /// returned marker expression is disjoint with another marker expression.
+    /// If it is, then one can conclude that the `Requires-Python` specifier
+    /// excludes the dependency with that other marker expression.
+    ///
+    /// If this `Requires-Python` specifier has no constraints, then this
+    /// returns a marker tree that evaluates to `true` for all possible marker
+    /// environments.
+    pub fn to_marker_tree(&self) -> MarkerTree {
+        let (op, version) = match self.bound.as_ref() {
+            // If we see this anywhere, then it implies the marker
+            // tree we would generate would always evaluate to
+            // `true` because every possible Python version would
+            // satisfy it.
+            Bound::Unbounded => return MarkerTree::And(vec![]),
+            Bound::Excluded(version) => (Operator::GreaterThan, version.clone().without_local()),
+            Bound::Included(version) => {
+                (Operator::GreaterThanEqual, version.clone().without_local())
+            }
+        };
+        // For the `python_version` marker expression, it specifically only
+        // supports truncate major/minor versions of Python. This means that
+        // a `Requires-Python: 3.10.1` is satisfied by `python_version ==
+        // '3.10'`. So for disjointness checking, we need to ensure that the
+        // marker expression we generate for `Requires-Python` doesn't try to
+        // be overly selective about the patch version. We do this by keeping
+        // this part of our marker limited to the major and minor version
+        // components only.
+        let version_major_minor_only = Version::new(version.release().iter().take(2));
+        let expr_python_version = MarkerExpression::Version {
+            key: MarkerValueVersion::PythonVersion,
+            // OK because a version specifier is only invalid when the
+            // version is local (which is impossible here because we
+            // strip it above) or if the operator is ~= (which is also
+            // impossible here).
+            specifier: VersionSpecifier::from_version(op, version_major_minor_only).unwrap(),
+        };
+        let expr_python_full_version = MarkerExpression::Version {
+            key: MarkerValueVersion::PythonFullVersion,
+            // For `python_full_version`, we can use the entire
+            // version as-is.
+            //
+            // OK because a version specifier is only invalid when the
+            // version is local (which is impossible here because we
+            // strip it above) or if the operator is ~= (which is also
+            // impossible here).
+            specifier: VersionSpecifier::from_version(op, version).unwrap(),
+        };
+        MarkerTree::And(vec![
+            MarkerTree::Expression(expr_python_version),
+            MarkerTree::Expression(expr_python_full_version),
+        ])
     }
 }
 
@@ -186,12 +259,50 @@ impl serde::Serialize for RequiresPython {
 impl<'de> serde::Deserialize<'de> for RequiresPython {
     fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
         let specifiers = VersionSpecifiers::deserialize(deserializer)?;
-        let bound = crate::pubgrub::PubGrubSpecifier::try_from(&specifiers)
-            .map_err(serde::de::Error::custom)?
-            .iter()
-            .next()
-            .map(|(lower, _)| lower.clone())
-            .unwrap_or(Bound::Unbounded);
+        let bound = RequiresPythonBound(
+            crate::pubgrub::PubGrubSpecifier::try_from(&specifiers)
+                .map_err(serde::de::Error::custom)?
+                .iter()
+                .next()
+                .map(|(lower, _)| lower.clone())
+                .unwrap_or(Bound::Unbounded),
+        );
         Ok(Self { specifiers, bound })
     }
 }
+
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+pub struct RequiresPythonBound(Bound<Version>);
+
+impl RequiresPythonBound {
+    pub fn new(bound: Bound<Version>) -> Self {
+        Self(bound)
+    }
+}
+
+impl Deref for RequiresPythonBound {
+    type Target = Bound<Version>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl PartialOrd for RequiresPythonBound {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for RequiresPythonBound {
+    fn cmp(&self, other: &Self) -> Ordering {
+        match (self.as_ref(), other.as_ref()) {
+            (Bound::Included(a), Bound::Included(b)) => a.cmp(b),
+            (Bound::Included(_), Bound::Excluded(_)) => Ordering::Less,
+            (Bound::Excluded(_), Bound::Included(_)) => Ordering::Greater,
+            (Bound::Excluded(a), Bound::Excluded(b)) => a.cmp(b),
+            (Bound::Unbounded, _) => Ordering::Less,
+            (_, Bound::Unbounded) => Ordering::Greater,
+        }
+    }
+}
