@@ -1,30 +1,31 @@
 use std::fmt::Write;
 
-use anyhow::{Context, Result};
 use itertools::Itertools;
 use owo_colors::OwoColorize;
 use tracing::debug;
 
-use distribution_types::Resolution;
+use distribution_types::{Resolution, UnresolvedRequirementSpecification};
 use pep440_rs::Version;
+use pypi_types::Requirement;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{Concurrency, ExtrasSpecification, PreviewMode, SetupPyStrategy};
 use uv_dispatch::BuildDispatch;
-use uv_distribution::Workspace;
+use uv_distribution::{DistributionDatabase, Workspace};
 use uv_fs::Simplified;
 use uv_git::GitResolver;
 use uv_installer::{SatisfiesResult, SitePackages};
-use uv_requirements::{RequirementsSource, RequirementsSpecification};
+use uv_requirements::{NamedRequirementsResolver, RequirementsSpecification};
 use uv_resolver::{FlatIndex, InMemoryIndex, OptionsBuilder, PythonRequirement, RequiresPython};
 use uv_toolchain::{
     request_from_version_file, EnvironmentPreference, Interpreter, PythonEnvironment, Toolchain,
-    ToolchainPreference, ToolchainRequest, VersionRequest,
+    ToolchainFetch, ToolchainPreference, ToolchainRequest, VersionRequest,
 };
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
-use uv_warnings::warn_user;
 
 use crate::commands::pip;
+use crate::commands::pip::operations::Modifications;
+use crate::commands::reporters::ResolverReporter;
 use crate::printer::Printer;
 use crate::settings::ResolverInstallerSettings;
 
@@ -36,11 +37,14 @@ pub(crate) mod sync;
 
 #[derive(thiserror::Error, Debug)]
 pub(crate) enum ProjectError {
-    #[error("The current Python version ({0}) is not compatible with the locked Python requirement ({1})")]
-    PythonIncompatibility(Version, RequiresPython),
+    #[error("The current Python version ({0}) is not compatible with the locked Python requirement: `{1}`")]
+    LockedPythonIncompatibility(Version, RequiresPython),
+
+    #[error("The requested Python interpreter ({0}) is incompatible with the project Python requirement: `{1}`")]
+    RequestedPythonIncompatibility(Version, RequiresPython),
 
     #[error(transparent)]
-    Interpreter(#[from] uv_toolchain::Error),
+    Toolchain(#[from] uv_toolchain::Error),
 
     #[error(transparent)]
     Virtualenv(#[from] uv_virtualenv::Error),
@@ -60,9 +64,6 @@ pub(crate) enum ProjectError {
     #[error(transparent)]
     Io(#[from] std::io::Error),
 
-    #[error(transparent)]
-    Serialize(#[from] toml::ser::Error),
-
     #[error(transparent)]
     Anyhow(#[from] anyhow::Error),
 
@@ -90,7 +91,7 @@ pub(crate) fn find_requires_python(
 }
 
 /// Find the virtual environment for the current project.
-pub(crate) fn find_environment(
+fn find_environment(
     workspace: &Workspace,
     cache: &Cache,
 ) -> Result<PythonEnvironment, uv_toolchain::Error> {
@@ -98,198 +99,293 @@ pub(crate) fn find_environment(
 }
 
 /// Check if the given interpreter satisfies the project's requirements.
-pub(crate) fn interpreter_meets_requirements(
+fn interpreter_meets_requirements(
     interpreter: &Interpreter,
     requested_python: Option<&ToolchainRequest>,
-    requires_python: Option<&RequiresPython>,
     cache: &Cache,
 ) -> bool {
-    // `--python` has highest precedence, after that we check `requires_python` from
-    // `pyproject.toml`. If `--python` and `requires_python` are mutually incompatible,
-    // we'll fail at the build or at last the install step when we aren't able to install
-    // the editable wheel for the current project into the venv.
-    // TODO(konsti): Do we want to support a workspace python version requirement?
-    if let Some(request) = requested_python {
-        if request.satisfied(interpreter, cache) {
-            debug!("Interpreter meets the requested python {}", request);
-            return true;
-        }
-
-        debug!("Interpreter does not meet the request {}", request);
-        return false;
+    let Some(request) = requested_python else {
+        return true;
     };
-
-    if let Some(requires_python) = requires_python {
-        if requires_python.contains(interpreter.python_version()) {
-            debug!("Interpreter meets the project `Requires-Python` constraint {requires_python}");
-            return true;
-        }
-
-        debug!(
-            "Interpreter does not meet the project `Requires-Python` constraint {requires_python}"
-        );
-        return false;
-    };
-
-    // No requirement to check
-    true
+    if request.satisfied(interpreter, cache) {
+        debug!("Interpreter meets the requested Python: `{request}`");
+        true
+    } else {
+        debug!("Interpreter does not meet the request: `{request}`");
+        false
+    }
 }
 
-/// Find the interpreter to use in the current project.
-pub(crate) async fn find_interpreter(
-    workspace: &Workspace,
-    python_request: Option<ToolchainRequest>,
-    toolchain_preference: ToolchainPreference,
-    connectivity: Connectivity,
-    native_tls: bool,
-    cache: &Cache,
-    printer: Printer,
-) -> Result<Interpreter, ProjectError> {
-    let requires_python = find_requires_python(workspace)?;
+#[derive(Debug)]
+pub(crate) enum FoundInterpreter {
+    Interpreter(Interpreter),
+    Environment(PythonEnvironment),
+}
 
-    // (1) Explicit request from user
-    let python_request = if let Some(request) = python_request {
-        Some(request)
-    // (2) Request from `.python-version`
-    } else if let Some(request) = request_from_version_file().await? {
-        Some(request)
-    // (3) `Requires-Python` in `pyproject.toml`
-    } else {
-        requires_python
-            .as_ref()
-            .map(RequiresPython::specifiers)
-            .map(|specifiers| ToolchainRequest::Version(VersionRequest::Range(specifiers.clone())))
-    };
+impl FoundInterpreter {
+    /// Discover the interpreter to use in the current [`Workspace`].
+    pub(crate) async fn discover(
+        workspace: &Workspace,
+        python_request: Option<ToolchainRequest>,
+        toolchain_preference: ToolchainPreference,
+        toolchain_fetch: ToolchainFetch,
+        connectivity: Connectivity,
+        native_tls: bool,
+        cache: &Cache,
+        printer: Printer,
+    ) -> Result<Self, ProjectError> {
+        let requires_python = find_requires_python(workspace)?;
 
-    // Read from the virtual environment first
-    match find_environment(workspace, cache) {
-        Ok(venv) => {
-            if interpreter_meets_requirements(
-                venv.interpreter(),
-                python_request.as_ref(),
-                requires_python.as_ref(),
-                cache,
-            ) {
-                return Ok(venv.into_interpreter());
+        // (1) Explicit request from user
+        let python_request = if let Some(request) = python_request {
+            Some(request)
+            // (2) Request from `.python-version`
+        } else if let Some(request) = request_from_version_file().await? {
+            Some(request)
+            // (3) `Requires-Python` in `pyproject.toml`
+        } else {
+            requires_python
+                .as_ref()
+                .map(RequiresPython::specifiers)
+                .map(|specifiers| {
+                    ToolchainRequest::Version(VersionRequest::Range(specifiers.clone()))
+                })
+        };
+
+        // Read from the virtual environment first.
+        match find_environment(workspace, cache) {
+            Ok(venv) => {
+                if interpreter_meets_requirements(
+                    venv.interpreter(),
+                    python_request.as_ref(),
+                    cache,
+                ) {
+                    if let Some(requires_python) = requires_python.as_ref() {
+                        if requires_python.contains(venv.interpreter().python_version()) {
+                            return Ok(Self::Environment(venv));
+                        }
+                        debug!(
+                            "Interpreter does not meet the project's Python requirement: `{requires_python}`"
+                        );
+                    } else {
+                        return Ok(Self::Environment(venv));
+                    }
+                }
+            }
+            Err(uv_toolchain::Error::MissingEnvironment(_)) => {}
+            Err(err) => return Err(err.into()),
+        };
+
+        let client_builder = BaseClientBuilder::default()
+            .connectivity(connectivity)
+            .native_tls(native_tls);
+
+        // Locate the Python interpreter to use in the environment
+        let interpreter = Toolchain::find_or_fetch(
+            python_request,
+            EnvironmentPreference::OnlySystem,
+            toolchain_preference,
+            toolchain_fetch,
+            &client_builder,
+            cache,
+        )
+        .await?
+        .into_interpreter();
+
+        writeln!(
+            printer.stderr(),
+            "Using Python {} interpreter at: {}",
+            interpreter.python_version(),
+            interpreter.sys_executable().user_display().cyan()
+        )?;
+
+        if let Some(requires_python) = requires_python.as_ref() {
+            if !requires_python.contains(interpreter.python_version()) {
+                return Err(ProjectError::RequestedPythonIncompatibility(
+                    interpreter.python_version().clone(),
+                    requires_python.clone(),
+                ));
             }
         }
-        Err(uv_toolchain::Error::MissingEnvironment(_)) => {}
-        Err(e) => return Err(e.into()),
-    };
 
-    let client_builder = BaseClientBuilder::default()
-        .connectivity(connectivity)
-        .native_tls(native_tls);
-
-    // Locate the Python interpreter to use in the environment
-    let interpreter = Toolchain::find_or_fetch(
-        python_request,
-        EnvironmentPreference::OnlySystem,
-        toolchain_preference,
-        client_builder,
-        cache,
-    )
-    .await?
-    .into_interpreter();
-
-    writeln!(
-        printer.stderr(),
-        "Using Python {} interpreter at: {}",
-        interpreter.python_version(),
-        interpreter.sys_executable().user_display().cyan()
-    )?;
-
-    if let Some(requires_python) = requires_python.as_ref() {
-        if !requires_python.contains(interpreter.python_version()) {
-            warn_user!(
-                "The Python interpreter ({}) is incompatible with the project Python requirement {}",
-                interpreter.python_version(),
-                requires_python
-            );
-        }
+        Ok(Self::Interpreter(interpreter))
     }
 
-    Ok(interpreter)
+    /// Convert the [`FoundInterpreter`] into an [`Interpreter`].
+    pub(crate) fn into_interpreter(self) -> Interpreter {
+        match self {
+            FoundInterpreter::Interpreter(interpreter) => interpreter,
+            FoundInterpreter::Environment(venv) => venv.into_interpreter(),
+        }
+    }
 }
 
 /// Initialize a virtual environment for the current project.
-pub(crate) async fn init_environment(
+pub(crate) async fn get_or_init_environment(
     workspace: &Workspace,
     python: Option<ToolchainRequest>,
     toolchain_preference: ToolchainPreference,
+    toolchain_fetch: ToolchainFetch,
     connectivity: Connectivity,
     native_tls: bool,
     cache: &Cache,
     printer: Printer,
 ) -> Result<PythonEnvironment, ProjectError> {
-    let requires_python = find_requires_python(workspace)?;
-
-    // Check if the environment exists and is sufficient
-    match find_environment(workspace, cache) {
-        Ok(venv) => {
-            if interpreter_meets_requirements(
-                venv.interpreter(),
-                python.as_ref(),
-                requires_python.as_ref(),
-                cache,
-            ) {
-                return Ok(venv);
-            }
-
-            // Remove the existing virtual environment if it doesn't meet the requirements
-            writeln!(
-                printer.stderr(),
-                "Removing virtual environment at: {}",
-                venv.root().user_display().cyan()
-            )?;
-            fs_err::remove_dir_all(venv.root())
-                .context("Failed to remove existing virtual environment")?;
-        }
-        Err(uv_toolchain::Error::MissingEnvironment(_)) => {}
-        Err(e) => return Err(e.into()),
-    };
-
-    // Find an interpreter to create the environment with
-    let interpreter = find_interpreter(
+    match FoundInterpreter::discover(
         workspace,
         python,
         toolchain_preference,
+        toolchain_fetch,
         connectivity,
         native_tls,
         cache,
         printer,
     )
-    .await?;
+    .await?
+    {
+        // If we found an existing, compatible environment, use it.
+        FoundInterpreter::Environment(environment) => Ok(environment),
 
-    let venv = workspace.venv();
-    writeln!(
-        printer.stderr(),
-        "Creating virtualenv at: {}",
-        venv.user_display().cyan()
-    )?;
+        // Otherwise, create a virtual environment with the discovered interpreter.
+        FoundInterpreter::Interpreter(interpreter) => {
+            let venv = workspace.venv();
 
-    Ok(uv_virtualenv::create_venv(
-        &venv,
-        interpreter,
-        uv_virtualenv::Prompt::None,
-        false,
-        false,
-    )?)
+            // Remove the existing virtual environment if it doesn't meet the requirements.
+            match fs_err::remove_dir_all(&venv) {
+                Ok(()) => {
+                    writeln!(
+                        printer.stderr(),
+                        "Removed virtual environment at: {}",
+                        venv.user_display().cyan()
+                    )?;
+                }
+                Err(e) if e.kind() == std::io::ErrorKind::NotFound => {}
+                Err(e) => return Err(e.into()),
+            }
+
+            writeln!(
+                printer.stderr(),
+                "Creating virtualenv at: {}",
+                venv.user_display().cyan()
+            )?;
+
+            Ok(uv_virtualenv::create_venv(
+                &venv,
+                interpreter,
+                uv_virtualenv::Prompt::None,
+                false,
+                false,
+            )?)
+        }
+    }
 }
 
-/// Update a [`PythonEnvironment`] to satisfy a set of [`RequirementsSource`]s.
-#[allow(clippy::too_many_arguments)]
-pub(crate) async fn update_environment(
-    venv: PythonEnvironment,
-    requirements: &[RequirementsSource],
+/// Shared state used during resolution and installation.
+#[derive(Default)]
+pub(crate) struct SharedState {
+    /// The resolved Git references.
+    git: GitResolver,
+    /// The fetched package versions and metadata.
+    index: InMemoryIndex,
+}
+
+/// Resolve any [`UnresolvedRequirementSpecification`] into a fully-qualified [`Requirement`].
+pub(crate) async fn resolve_names(
+    requirements: Vec<UnresolvedRequirementSpecification>,
+    interpreter: &Interpreter,
     settings: &ResolverInstallerSettings,
+    state: &SharedState,
     preview: PreviewMode,
     connectivity: Connectivity,
     concurrency: Concurrency,
     native_tls: bool,
     cache: &Cache,
     printer: Printer,
-) -> Result<PythonEnvironment> {
+) -> anyhow::Result<Vec<Requirement>> {
+    // Extract the project settings.
+    let ResolverInstallerSettings {
+        index_locations,
+        index_strategy,
+        keyring_provider,
+        resolution: _,
+        prerelease: _,
+        config_setting,
+        exclude_newer,
+        link_mode,
+        compile_bytecode: _,
+        upgrade: _,
+        reinstall: _,
+        build_options,
+    } = settings;
+
+    // Initialize the registry client.
+    let client = RegistryClientBuilder::new(cache.clone())
+        .native_tls(native_tls)
+        .connectivity(connectivity)
+        .index_urls(index_locations.index_urls())
+        .index_strategy(*index_strategy)
+        .keyring(*keyring_provider)
+        .markers(interpreter.markers())
+        .platform(interpreter.platform())
+        .build();
+
+    // Initialize any shared state.
+    let in_flight = InFlight::default();
+
+    // TODO(charlie): These are all default values. We should consider whether we want to make them
+    // optional on the downstream APIs.
+    let build_isolation = BuildIsolation::default();
+    let hasher = HashStrategy::default();
+    let setup_py = SetupPyStrategy::default();
+    let flat_index = FlatIndex::default();
+
+    // Create a build dispatch.
+    let build_dispatch = BuildDispatch::new(
+        &client,
+        cache,
+        interpreter,
+        index_locations,
+        &flat_index,
+        &state.index,
+        &state.git,
+        &in_flight,
+        *index_strategy,
+        setup_py,
+        config_setting,
+        build_isolation,
+        *link_mode,
+        build_options,
+        *exclude_newer,
+        concurrency,
+        preview,
+    );
+
+    // Initialize the resolver.
+    let resolver = NamedRequirementsResolver::new(
+        requirements,
+        &hasher,
+        &state.index,
+        DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads, preview),
+    )
+    .with_reporter(ResolverReporter::from(printer));
+
+    Ok(resolver.resolve().await?)
+}
+
+/// Update a [`PythonEnvironment`] to satisfy a set of [`RequirementsSource`]s.
+pub(crate) async fn update_environment(
+    venv: PythonEnvironment,
+    spec: RequirementsSpecification,
+    modifications: Modifications,
+    settings: &ResolverInstallerSettings,
+    state: &SharedState,
+    preview: PreviewMode,
+    connectivity: Connectivity,
+    concurrency: Concurrency,
+    native_tls: bool,
+    cache: &Cache,
+    printer: Printer,
+) -> anyhow::Result<PythonEnvironment> {
     // Extract the project settings.
     let ResolverInstallerSettings {
         index_locations,
@@ -306,20 +402,9 @@ pub(crate) async fn update_environment(
         build_options,
     } = settings;
 
-    let client_builder = BaseClientBuilder::new()
-        .connectivity(connectivity)
-        .native_tls(native_tls)
-        .keyring(*keyring_provider);
-
-    // Read all requirements from the provided sources.
-    // TODO(zanieb): Consider allowing constraints and extras
-    // TODO(zanieb): Allow specifying extras somehow
-    let spec =
-        RequirementsSpecification::from_sources(requirements, &[], &[], &client_builder).await?;
-
     // Check if the current environment satisfies the requirements
     let site_packages = SitePackages::from_environment(&venv)?;
-    if spec.source_trees.is_empty() && reinstall.is_none() {
+    if spec.source_trees.is_empty() && reinstall.is_none() && upgrade.is_none() {
         match site_packages.satisfies(&spec.requirements, &spec.constraints)? {
             // If the requirements are already satisfied, we're done.
             SatisfiesResult::Fresh {
@@ -366,9 +451,7 @@ pub(crate) async fn update_environment(
         .build();
 
     // Initialize any shared state.
-    let git = GitResolver::default();
     let in_flight = InFlight::default();
-    let index = InMemoryIndex::default();
 
     // TODO(charlie): These are all default values. We should consider whether we want to make them
     // optional on the downstream APIs.
@@ -394,8 +477,8 @@ pub(crate) async fn update_environment(
         interpreter,
         index_locations,
         &flat_index,
-        &index,
-        &git,
+        &state.index,
+        &state.git,
         &in_flight,
         *index_strategy,
         setup_py,
@@ -427,7 +510,7 @@ pub(crate) async fn update_environment(
         python_requirement,
         &client,
         &flat_index,
-        &index,
+        &state.index,
         &resolve_dispatch,
         concurrency,
         options,
@@ -454,8 +537,8 @@ pub(crate) async fn update_environment(
             interpreter,
             index_locations,
             &flat_index,
-            &index,
-            &git,
+            &state.index,
+            &state.git,
             &in_flight,
             *index_strategy,
             setup_py,
@@ -473,7 +556,7 @@ pub(crate) async fn update_environment(
     pip::operations::install(
         &resolution,
         site_packages,
-        pip::operations::Modifications::Sufficient,
+        modifications,
         reinstall,
         build_options,
         *link_mode,
