@@ -8,10 +8,10 @@ use uv_configuration::PreviewMode;
 use uv_fs::Simplified;
 use uv_toolchain::downloads::PythonDownloadRequest;
 use uv_toolchain::{
-    find_toolchains, DiscoveryError, EnvironmentPreference, Toolchain, ToolchainNotFound,
-    ToolchainPreference, ToolchainRequest, ToolchainSource,
+    find_toolchains, DiscoveryError, EnvironmentPreference, Toolchain, ToolchainFetch,
+    ToolchainNotFound, ToolchainPreference, ToolchainRequest, ToolchainSource,
 };
-use uv_warnings::warn_user;
+use uv_warnings::warn_user_once;
 
 use crate::commands::ExitStatus;
 use crate::printer::Printer;
@@ -31,21 +31,29 @@ pub(crate) async fn list(
     all_versions: bool,
     all_platforms: bool,
     toolchain_preference: ToolchainPreference,
+    toolchain_fetch: ToolchainFetch,
     preview: PreviewMode,
     cache: &Cache,
     printer: Printer,
 ) -> Result<ExitStatus> {
     if preview.is_disabled() {
-        warn_user!("`uv toolchain list` is experimental and may change without warning.");
+        warn_user_once!("`uv toolchain list` is experimental and may change without warning.");
     }
 
     let download_request = match kinds {
         ToolchainListKinds::Installed => None,
-        ToolchainListKinds::Default => Some(if all_platforms {
-            PythonDownloadRequest::default()
-        } else {
-            PythonDownloadRequest::from_env()?
-        }),
+        ToolchainListKinds::Default => {
+            if toolchain_fetch.is_automatic() {
+                Some(if all_platforms {
+                    PythonDownloadRequest::default()
+                } else {
+                    PythonDownloadRequest::from_env()?
+                })
+            } else {
+                // If fetching is not automatic, then don't show downloads as available by default
+                None
+            }
+        }
     };
 
     let downloads = download_request
