@@ -6,15 +6,16 @@ use std::str::FromStr;
 
 use distribution_types::IndexLocations;
 use install_wheel_rs::linker::LinkMode;
-use pep508_rs::RequirementOrigin;
+use pep508_rs::{ExtraName, RequirementOrigin};
 use pypi_types::Requirement;
 use uv_cache::{CacheArgs, Refresh};
 use uv_cli::options::{flag, installer_options, resolver_installer_options, resolver_options};
 use uv_cli::{
     AddArgs, ColorChoice, Commands, ExternalCommand, GlobalArgs, ListFormat, LockArgs, Maybe,
     PipCheckArgs, PipCompileArgs, PipFreezeArgs, PipInstallArgs, PipListArgs, PipShowArgs,
-    PipSyncArgs, PipTreeArgs, PipUninstallArgs, RemoveArgs, RunArgs, SyncArgs, ToolRunArgs,
-    ToolchainFindArgs, ToolchainInstallArgs, ToolchainListArgs, VenvArgs,
+    PipSyncArgs, PipTreeArgs, PipUninstallArgs, RemoveArgs, RunArgs, SyncArgs, ToolInstallArgs,
+    ToolListArgs, ToolRunArgs, ToolUninstallArgs, ToolchainFindArgs, ToolchainInstallArgs,
+    ToolchainListArgs, ToolchainUninstallArgs, VenvArgs,
 };
 use uv_client::Connectivity;
 use uv_configuration::{
@@ -22,6 +23,7 @@ use uv_configuration::{
     KeyringProviderType, NoBinary, NoBuild, PreviewMode, Reinstall, SetupPyStrategy, TargetTriple,
     Upgrade,
 };
+use uv_distribution::pyproject::DependencyType;
 use uv_normalize::PackageName;
 use uv_requirements::RequirementsSource;
 use uv_resolver::{AnnotationStyle, DependencyMode, ExcludeNewer, PreReleaseMode, ResolutionMode};
@@ -29,7 +31,7 @@ use uv_settings::{
     Combine, FilesystemOptions, InstallerOptions, Options, PipOptions, ResolverInstallerOptions,
     ResolverOptions,
 };
-use uv_toolchain::{Prefix, PythonVersion, Target, ToolchainPreference};
+use uv_toolchain::{Prefix, PythonVersion, Target, ToolchainFetch, ToolchainPreference};
 
 use crate::commands::pip::operations::Modifications;
 
@@ -46,6 +48,7 @@ pub(crate) struct GlobalSettings {
     pub(crate) show_settings: bool,
     pub(crate) preview: PreviewMode,
     pub(crate) toolchain_preference: ToolchainPreference,
+    pub(crate) toolchain_fetch: ToolchainFetch,
 }
 
 impl GlobalSettings {
@@ -112,6 +115,10 @@ impl GlobalSettings {
                 .toolchain_preference
                 .combine(workspace.and_then(|workspace| workspace.globals.toolchain_preference))
                 .unwrap_or(default_toolchain_preference),
+            toolchain_fetch: args
+                .toolchain_fetch
+                .combine(workspace.and_then(|workspace| workspace.globals.toolchain_fetch))
+                .unwrap_or_default(),
         }
     }
 }
@@ -147,8 +154,8 @@ pub(crate) struct RunSettings {
     pub(crate) dev: bool,
     pub(crate) command: ExternalCommand,
     pub(crate) with: Vec<String>,
-    pub(crate) python: Option<String>,
     pub(crate) package: Option<PackageName>,
+    pub(crate) python: Option<String>,
     pub(crate) refresh: Refresh,
     pub(crate) settings: ResolverInstallerSettings,
 }
@@ -168,8 +175,8 @@ impl RunSettings {
             installer,
             build,
             refresh,
-            python,
             package,
+            python,
         } = args;
 
         Self {
@@ -180,8 +187,8 @@ impl RunSettings {
             dev: flag(dev, no_dev).unwrap_or(true),
             command,
             with,
-            python,
             package,
+            python,
             refresh: Refresh::from(refresh),
             settings: ResolverInstallerSettings::combine(
                 resolver_installer_options(installer, build),
@@ -231,6 +238,81 @@ impl ToolRunSettings {
     }
 }
 
+/// The resolved settings to use for a `tool install` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct ToolInstallSettings {
+    pub(crate) package: String,
+    pub(crate) from: Option<String>,
+    pub(crate) with: Vec<String>,
+    pub(crate) python: Option<String>,
+    pub(crate) refresh: Refresh,
+    pub(crate) settings: ResolverInstallerSettings,
+    pub(crate) force: bool,
+}
+
+impl ToolInstallSettings {
+    /// Resolve the [`ToolInstallSettings`] from the CLI and filesystem configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(args: ToolInstallArgs, filesystem: Option<FilesystemOptions>) -> Self {
+        let ToolInstallArgs {
+            package,
+            from,
+            with,
+            installer,
+            force,
+            build,
+            refresh,
+            python,
+        } = args;
+
+        Self {
+            package,
+            from,
+            with,
+            python,
+            force,
+            refresh: Refresh::from(refresh),
+            settings: ResolverInstallerSettings::combine(
+                resolver_installer_options(installer, build),
+                filesystem,
+            ),
+        }
+    }
+}
+
+/// The resolved settings to use for a `tool list` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct ToolListSettings;
+
+impl ToolListSettings {
+    /// Resolve the [`ToolListSettings`] from the CLI and filesystem configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(args: ToolListArgs, _filesystem: Option<FilesystemOptions>) -> Self {
+        let ToolListArgs {} = args;
+
+        Self {}
+    }
+}
+
+/// The resolved settings to use for a `tool uninstall` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct ToolUninstallSettings {
+    pub(crate) name: PackageName,
+}
+
+impl ToolUninstallSettings {
+    /// Resolve the [`ToolUninstallSettings`] from the CLI and filesystem configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(args: ToolUninstallArgs, _filesystem: Option<FilesystemOptions>) -> Self {
+        let ToolUninstallArgs { name } = args;
+
+        Self { name }
+    }
+}
+
 #[derive(Debug, Clone, Default)]
 pub(crate) enum ToolchainListKinds {
     #[default]
@@ -292,6 +374,26 @@ impl ToolchainInstallSettings {
     }
 }
 
+/// The resolved settings to use for a `toolchain uninstall` invocation.
+#[allow(clippy::struct_excessive_bools)]
+#[derive(Debug, Clone)]
+pub(crate) struct ToolchainUninstallSettings {
+    pub(crate) targets: Vec<String>,
+}
+
+impl ToolchainUninstallSettings {
+    /// Resolve the [`ToolchainUninstallSettings`] from the CLI and filesystem configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(
+        args: ToolchainUninstallArgs,
+        _filesystem: Option<FilesystemOptions>,
+    ) -> Self {
+        let ToolchainUninstallArgs { targets } = args;
+
+        Self { targets }
+    }
+}
+
 /// The resolved settings to use for a `toolchain find` invocation.
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
@@ -391,13 +493,14 @@ impl LockSettings {
 #[derive(Debug, Clone)]
 pub(crate) struct AddSettings {
     pub(crate) requirements: Vec<RequirementsSource>,
-    pub(crate) dev: bool,
-    pub(crate) workspace: bool,
+    pub(crate) dependency_type: DependencyType,
     pub(crate) editable: Option<bool>,
-    pub(crate) raw: bool,
+    pub(crate) extras: Vec<ExtraName>,
+    pub(crate) raw_sources: bool,
     pub(crate) rev: Option<String>,
     pub(crate) tag: Option<String>,
     pub(crate) branch: Option<String>,
+    pub(crate) package: Option<PackageName>,
     pub(crate) python: Option<String>,
     pub(crate) refresh: Refresh,
     pub(crate) settings: ResolverInstallerSettings,
@@ -410,15 +513,17 @@ impl AddSettings {
         let AddArgs {
             requirements,
             dev,
-            workspace,
+            optional,
             editable,
-            raw,
+            extra,
+            raw_sources,
             rev,
             tag,
             branch,
             installer,
             build,
             refresh,
+            package,
             python,
         } = args;
 
@@ -427,16 +532,25 @@ impl AddSettings {
             .map(RequirementsSource::Package)
             .collect::<Vec<_>>();
 
+        let dependency_type = if let Some(group) = optional {
+            DependencyType::Optional(group)
+        } else if dev {
+            DependencyType::Dev
+        } else {
+            DependencyType::Production
+        };
+
         Self {
             requirements,
-            workspace,
-            dev,
+            dependency_type,
             editable,
-            raw,
+            raw_sources,
             rev,
             tag,
             branch,
+            package,
             python,
+            extras: extra.unwrap_or_default(),
             refresh: Refresh::from(refresh),
             settings: ResolverInstallerSettings::combine(
                 resolver_installer_options(installer, build),
@@ -451,7 +565,8 @@ impl AddSettings {
 #[derive(Debug, Clone)]
 pub(crate) struct RemoveSettings {
     pub(crate) requirements: Vec<PackageName>,
-    pub(crate) dev: bool,
+    pub(crate) dependency_type: DependencyType,
+    pub(crate) package: Option<PackageName>,
     pub(crate) python: Option<String>,
 }
 
@@ -461,13 +576,24 @@ impl RemoveSettings {
     pub(crate) fn resolve(args: RemoveArgs, _filesystem: Option<FilesystemOptions>) -> Self {
         let RemoveArgs {
             dev,
+            optional,
             requirements,
+            package,
             python,
         } = args;
 
+        let dependency_type = if let Some(group) = optional {
+            DependencyType::Optional(group)
+        } else if dev {
+            DependencyType::Dev
+        } else {
+            DependencyType::Production
+        };
+
         Self {
             requirements,
-            dev,
+            dependency_type,
+            package,
             python,
         }
     }
@@ -501,6 +627,8 @@ impl PipCompileSettings {
             output_file,
             no_strip_extras,
             strip_extras,
+            no_strip_markers,
+            strip_markers,
             no_annotate,
             annotate,
             no_header,
@@ -523,6 +651,8 @@ impl PipCompileSettings {
             only_binary,
             python_version,
             python_platform,
+            universal,
+            no_universal,
             no_emit_package,
             emit_index_url,
             no_emit_index_url,
@@ -573,6 +703,7 @@ impl PipCompileSettings {
                     no_deps: flag(no_deps, deps),
                     output_file,
                     no_strip_extras: flag(no_strip_extras, strip_extras),
+                    no_strip_markers: flag(no_strip_markers, strip_markers),
                     no_annotate: flag(no_annotate, annotate),
                     no_header: flag(no_header, header),
                     custom_compile_command,
@@ -580,6 +711,7 @@ impl PipCompileSettings {
                     legacy_setup_py: flag(legacy_setup_py, no_legacy_setup_py),
                     python_version,
                     python_platform,
+                    universal: flag(universal, no_universal),
                     no_emit_package,
                     emit_index_url: flag(emit_index_url, no_emit_index_url),
                     emit_find_links: flag(emit_find_links, no_emit_find_links),
@@ -627,6 +759,8 @@ impl PipSyncSettings {
             no_break_system_packages,
             target,
             prefix,
+            allow_empty_requirements,
+            no_allow_empty_requirements,
             legacy_setup_py,
             no_legacy_setup_py,
             no_build_isolation,
@@ -659,15 +793,19 @@ impl PipSyncSettings {
                     exclude_newer,
                     target,
                     prefix,
+                    require_hashes: flag(require_hashes, no_require_hashes),
                     no_build: flag(no_build, build),
                     no_binary,
                     only_binary,
-                    no_build_isolation: flag(no_build_isolation, build_isolation),
-                    strict: flag(strict, no_strict),
+                    allow_empty_requirements: flag(
+                        allow_empty_requirements,
+                        no_allow_empty_requirements,
+                    ),
                     legacy_setup_py: flag(legacy_setup_py, no_legacy_setup_py),
+                    no_build_isolation: flag(no_build_isolation, build_isolation),
                     python_version,
                     python_platform,
-                    require_hashes: flag(require_hashes, no_require_hashes),
+                    strict: flag(strict, no_strict),
                     concurrent_builds: env(env::CONCURRENT_BUILDS),
                     concurrent_downloads: env(env::CONCURRENT_DOWNLOADS),
                     concurrent_installs: env(env::CONCURRENT_INSTALLS),
@@ -814,6 +952,7 @@ impl PipUninstallSettings {
             no_break_system_packages,
             target,
             prefix,
+            compat_args: _,
         } = args;
 
         Self {
@@ -853,6 +992,7 @@ impl PipFreezeSettings {
             python,
             system,
             no_system,
+            compat_args: _,
         } = args;
 
         Self {
@@ -933,6 +1073,7 @@ impl PipShowSettings {
             python,
             system,
             no_system,
+            compat_args: _,
         } = args;
 
         Self {
@@ -954,6 +1095,11 @@ impl PipShowSettings {
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Debug, Clone)]
 pub(crate) struct PipTreeSettings {
+    pub(crate) depth: u8,
+    pub(crate) prune: Vec<PackageName>,
+    pub(crate) package: Vec<PackageName>,
+    pub(crate) no_dedupe: bool,
+    pub(crate) invert: bool,
     // CLI-only settings.
     pub(crate) shared: PipSettings,
 }
@@ -962,14 +1108,25 @@ impl PipTreeSettings {
     /// Resolve the [`PipTreeSettings`] from the CLI and workspace configuration.
     pub(crate) fn resolve(args: PipTreeArgs, filesystem: Option<FilesystemOptions>) -> Self {
         let PipTreeArgs {
+            depth,
+            prune,
+            package,
+            no_dedupe,
+            invert,
             strict,
             no_strict,
             python,
             system,
             no_system,
+            compat_args: _,
         } = args;
 
         Self {
+            depth,
+            prune,
+            package,
+            no_dedupe,
+            invert,
             // Shared settings.
             shared: PipSettings::combine(
                 PipOptions {
@@ -1084,6 +1241,18 @@ pub(crate) struct InstallerSettings {
     pub(crate) build_options: BuildOptions,
 }
 
+#[derive(Debug, Clone)]
+pub(crate) struct InstallerSettingsRef<'a> {
+    pub(crate) index_locations: &'a IndexLocations,
+    pub(crate) index_strategy: IndexStrategy,
+    pub(crate) keyring_provider: KeyringProviderType,
+    pub(crate) config_setting: &'a ConfigSettings,
+    pub(crate) link_mode: LinkMode,
+    pub(crate) compile_bytecode: bool,
+    pub(crate) reinstall: &'a Reinstall,
+    pub(crate) build_options: &'a BuildOptions,
+}
+
 impl InstallerSettings {
     /// Resolve the [`InstallerSettings`] from the CLI and filesystem configuration.
     pub(crate) fn combine(args: InstallerOptions, filesystem: Option<FilesystemOptions>) -> Self {
@@ -1161,6 +1330,19 @@ impl InstallerSettings {
             ),
         }
     }
+
+    pub(crate) fn as_ref(&self) -> InstallerSettingsRef {
+        InstallerSettingsRef {
+            index_locations: &self.index_locations,
+            index_strategy: self.index_strategy,
+            keyring_provider: self.keyring_provider,
+            config_setting: &self.config_setting,
+            link_mode: self.link_mode,
+            compile_bytecode: self.compile_bytecode,
+            reinstall: &self.reinstall,
+            build_options: &self.build_options,
+        }
+    }
 }
 
 /// The resolved settings to use for an invocation of the `uv` CLI when resolving dependencies.
@@ -1182,6 +1364,20 @@ pub(crate) struct ResolverSettings {
     pub(crate) build_options: BuildOptions,
 }
 
+#[derive(Debug, Clone)]
+pub(crate) struct ResolverSettingsRef<'a> {
+    pub(crate) index_locations: &'a IndexLocations,
+    pub(crate) index_strategy: IndexStrategy,
+    pub(crate) keyring_provider: KeyringProviderType,
+    pub(crate) resolution: ResolutionMode,
+    pub(crate) prerelease: PreReleaseMode,
+    pub(crate) config_setting: &'a ConfigSettings,
+    pub(crate) exclude_newer: Option<ExcludeNewer>,
+    pub(crate) link_mode: LinkMode,
+    pub(crate) upgrade: &'a Upgrade,
+    pub(crate) build_options: &'a BuildOptions,
+}
+
 impl ResolverSettings {
     /// Resolve the [`ResolverSettings`] from the CLI and filesystem configuration.
     pub(crate) fn combine(args: ResolverOptions, filesystem: Option<FilesystemOptions>) -> Self {
@@ -1258,6 +1454,21 @@ impl ResolverSettings {
             ),
         }
     }
+
+    pub(crate) fn as_ref(&self) -> ResolverSettingsRef {
+        ResolverSettingsRef {
+            index_locations: &self.index_locations,
+            index_strategy: self.index_strategy,
+            keyring_provider: self.keyring_provider,
+            resolution: self.resolution,
+            prerelease: self.prerelease,
+            config_setting: &self.config_setting,
+            exclude_newer: self.exclude_newer,
+            link_mode: self.link_mode,
+            upgrade: &self.upgrade,
+            build_options: &self.build_options,
+        }
+    }
 }
 
 /// The resolved settings to use for an invocation of the `uv` CLI with both resolver and installer
@@ -1282,6 +1493,22 @@ pub(crate) struct ResolverInstallerSettings {
     pub(crate) build_options: BuildOptions,
 }
 
+#[derive(Debug, Clone)]
+pub(crate) struct ResolverInstallerSettingsRef<'a> {
+    pub(crate) index_locations: &'a IndexLocations,
+    pub(crate) index_strategy: IndexStrategy,
+    pub(crate) keyring_provider: KeyringProviderType,
+    pub(crate) resolution: ResolutionMode,
+    pub(crate) prerelease: PreReleaseMode,
+    pub(crate) config_setting: &'a ConfigSettings,
+    pub(crate) exclude_newer: Option<ExcludeNewer>,
+    pub(crate) link_mode: LinkMode,
+    pub(crate) compile_bytecode: bool,
+    pub(crate) upgrade: &'a Upgrade,
+    pub(crate) reinstall: &'a Reinstall,
+    pub(crate) build_options: &'a BuildOptions,
+}
+
 impl ResolverInstallerSettings {
     /// Resolve the [`ResolverInstallerSettings`] from the CLI and filesystem configuration.
     pub(crate) fn combine(
@@ -1371,6 +1598,23 @@ impl ResolverInstallerSettings {
             ),
         }
     }
+
+    pub(crate) fn as_ref(&self) -> ResolverInstallerSettingsRef {
+        ResolverInstallerSettingsRef {
+            index_locations: &self.index_locations,
+            index_strategy: self.index_strategy,
+            keyring_provider: self.keyring_provider,
+            resolution: self.resolution,
+            prerelease: self.prerelease,
+            config_setting: &self.config_setting,
+            exclude_newer: self.exclude_newer,
+            link_mode: self.link_mode,
+            compile_bytecode: self.compile_bytecode,
+            upgrade: &self.upgrade,
+            reinstall: &self.reinstall,
+            build_options: &self.build_options,
+        }
+    }
 }
 
 /// The resolved settings to use for an invocation of the `pip` CLI.
@@ -1391,12 +1635,14 @@ pub(crate) struct PipSettings {
     pub(crate) keyring_provider: KeyringProviderType,
     pub(crate) no_build_isolation: bool,
     pub(crate) build_options: BuildOptions,
+    pub(crate) allow_empty_requirements: bool,
     pub(crate) strict: bool,
     pub(crate) dependency_mode: DependencyMode,
     pub(crate) resolution: ResolutionMode,
     pub(crate) prerelease: PreReleaseMode,
     pub(crate) output_file: Option<PathBuf>,
     pub(crate) no_strip_extras: bool,
+    pub(crate) no_strip_markers: bool,
     pub(crate) no_annotate: bool,
     pub(crate) no_header: bool,
     pub(crate) custom_compile_command: Option<String>,
@@ -1405,6 +1651,7 @@ pub(crate) struct PipSettings {
     pub(crate) config_setting: ConfigSettings,
     pub(crate) python_version: Option<PythonVersion>,
     pub(crate) python_platform: Option<TargetTriple>,
+    pub(crate) universal: bool,
     pub(crate) exclude_newer: Option<ExcludeNewer>,
     pub(crate) no_emit_package: Vec<PackageName>,
     pub(crate) emit_index_url: bool,
@@ -1448,10 +1695,12 @@ impl PipSettings {
             extra,
             all_extras,
             no_deps,
+            allow_empty_requirements,
             resolution,
             prerelease,
             output_file,
             no_strip_extras,
+            no_strip_markers,
             no_annotate,
             no_header,
             custom_compile_command,
@@ -1460,6 +1709,7 @@ impl PipSettings {
             config_settings,
             python_version,
             python_platform,
+            universal,
             exclude_newer,
             no_emit_package,
             emit_index_url,
@@ -1549,6 +1799,10 @@ impl PipSettings {
                 .no_strip_extras
                 .combine(no_strip_extras)
                 .unwrap_or_default(),
+            no_strip_markers: args
+                .no_strip_markers
+                .combine(no_strip_markers)
+                .unwrap_or_default(),
             no_annotate: args.no_annotate.combine(no_annotate).unwrap_or_default(),
             no_header: args.no_header.combine(no_header).unwrap_or_default(),
             custom_compile_command: args.custom_compile_command.combine(custom_compile_command),
@@ -1568,6 +1822,10 @@ impl PipSettings {
                 .generate_hashes
                 .combine(generate_hashes)
                 .unwrap_or_default(),
+            allow_empty_requirements: args
+                .allow_empty_requirements
+                .combine(allow_empty_requirements)
+                .unwrap_or_default(),
             setup_py: if args
                 .legacy_setup_py
                 .combine(legacy_setup_py)
@@ -1587,6 +1845,7 @@ impl PipSettings {
                 .unwrap_or_default(),
             python_version: args.python_version.combine(python_version),
             python_platform: args.python_platform.combine(python_platform),
+            universal: args.universal.combine(universal).unwrap_or_default(),
             exclude_newer: args.exclude_newer.combine(exclude_newer),
             no_emit_package: args
                 .no_emit_package
@@ -1678,6 +1937,38 @@ impl PipSettings {
     }
 }
 
+impl<'a> From<ResolverInstallerSettingsRef<'a>> for ResolverSettingsRef<'a> {
+    fn from(settings: ResolverInstallerSettingsRef<'a>) -> Self {
+        Self {
+            index_locations: settings.index_locations,
+            index_strategy: settings.index_strategy,
+            keyring_provider: settings.keyring_provider,
+            resolution: settings.resolution,
+            prerelease: settings.prerelease,
+            config_setting: settings.config_setting,
+            exclude_newer: settings.exclude_newer,
+            link_mode: settings.link_mode,
+            upgrade: settings.upgrade,
+            build_options: settings.build_options,
+        }
+    }
+}
+
+impl<'a> From<ResolverInstallerSettingsRef<'a>> for InstallerSettingsRef<'a> {
+    fn from(settings: ResolverInstallerSettingsRef<'a>) -> Self {
+        Self {
+            index_locations: settings.index_locations,
+            index_strategy: settings.index_strategy,
+            keyring_provider: settings.keyring_provider,
+            config_setting: settings.config_setting,
+            link_mode: settings.link_mode,
+            compile_bytecode: settings.compile_bytecode,
+            reinstall: settings.reinstall,
+            build_options: settings.build_options,
+        }
+    }
+}
+
 // Environment variables that are not exposed as CLI arguments.
 mod env {
     pub(super) const CONCURRENT_DOWNLOADS: (&str, &str) =
